/* vim: set sts=4 ts=4 sw=4 et nojs co=88 tw=87: */
using System;
using System.Collections;

using RoBTreeFast = RobbieUtilities.Algorithms.BTreeFast.BTree;
using RoBTreeFastKeyDivPos = RobbieUtilities.Algorithms.BTreeFast.
    BTreePartialKeyDivPos;
using RoBTree = RobbieUtilities.Algorithms.BTree.BTree;
using RoBTreeKeyDivPos = RobbieUtilities.Algorithms.BTree.BTreePartialKeyDivPos;

/**
 *  RoBTreeFastTest:
 *      This is just a simple test, not meant to be comprehensive.
 */
namespace RobbieTests
{
    public class RoBTreeFastTest
    {
        public static void Main()
        {
            int[] arr = new int[] { 7, 8, 13, 19, 21, 30, 32, 35, 48, 57, 63, 66, 90 };
            RoBTreeFast btree1 = new RoBTreeFast(201);
            IEnumerator enum1;
            RoBTree btree2 = new RoBTree(201, new AutoComparer());
            DateTime time1;
            int bignum = 1000000;
            int i;

            for (i = 0; i < arr.Length; i++) {
                btree1.InsertElement(arr[i]);
            }
            enum1 = btree1.GetEnumerByPartialKey(new TestPKComparer(10), 30,
                RoBTreeFastKeyDivPos.KeyBeg, RoBTreeFastKeyDivPos.KeyEnd, true);
            while (enum1.MoveNext()) {
                Console.Write("{0} ", enum1.Current);
            }
            Console.WriteLine();
            for (i = 0; i < arr.Length; i++) {
                btree2.InsertElement(arr[i]);
            }
            enum1 = btree2.GetEnumerByPartialKey(new TestPKComparer(10), 60,
                RoBTreeKeyDivPos.KeyBeg, RoBTreeKeyDivPos.KeyEnd, true);
            while (enum1.MoveNext()) {
                Console.Write("{0} ", enum1.Current);
            }
            Console.WriteLine();
            time1 = DateTime.Now;
            Console.WriteLine("Inserting {0} integers...", bignum);
            for (i = 0; i < bignum; i++) {
                btree2.InsertElement(i);
            }
            Console.WriteLine("Deleting {0} integers...", bignum);
            for (i = 0; i < bignum; i++) {
                btree2.DeleteElement(i);
            }
            Console.WriteLine("{0} seconds", (DateTime.Now - time1).TotalSeconds);
            time1 = DateTime.Now;
            Console.WriteLine("Inserting {0} integers...", bignum);
            for (i = 0; i < bignum; i++) {
                btree1.InsertElement(i);
            }
            Console.WriteLine("Deleting {0} integers...", bignum);
            for (i = 0; i < bignum; i++) {
                btree1.DeleteElement(i);
            }
            Console.WriteLine("{0} seconds", (DateTime.Now - time1).TotalSeconds);
        }

        public class AutoComparer : IComparer
        {
            public AutoComparer()
            {
            }

            public int Compare(object oa, object ob)
            {
                return ((IComparable)oa).CompareTo(ob);
            }
        }

        public class TestPKComparer : IComparer
        {
            private int m_divisor;

            public TestPKComparer(int divisor)
            {
                m_divisor = divisor;
            }

            public int Compare(object oa, object ob)
            {
                return ((int)oa / m_divisor).CompareTo((int)ob / m_divisor);
            }
        }
    }
}
